"""
数据中心-直播
"""

from enum import Enum
from os import path
from tempfile import gettempdir
from typing import Callable

from BrowserAutomationLauncher import Browser, DataPacketProcessor
from BrowserAutomationLauncher._utils.tools import DictUtils, OsTools
from DrissionPage._pages.mix_tab import MixTab

from ._utils import pick__daterange


class Urls:
    live_list = 'https://ark.xiaohongshu.com/app-datacenter/live-list'


class DataPacketUrls:
    data_query_base = 'https://ark.xiaohongshu.com/api/edith/butterfly/data'
    """数据查询统一接口, 通过 type 参数区分查询的数据"""
    download_task = 'https://ark.xiaohongshu.com/api/edith/long_task/task/detail'
    """下载任务接口"""


class DataPacketType(Enum):
    LIVE_LIST_DETAIL = 'sellerLiveDetailData'
    """直播间详情列表"""


class Live:
    def __init__(self, browser: Browser):
        self._browser = browser
        self._timeout = 15

    def _wait__data_query(
        self, page: MixTab, data_type: str, func: Callable, timeout: float = None
    ):
        """
        等待统一数据查询接口数据包返回

        Args:
            page: 操作的页面对象
            data_type: 查询的数据类型
            func: 数据捕获之前需要执行的方法
            timeout: 超时时间
        """

        _timeout = timeout if isinstance(timeout, (float, int)) else self._timeout

        page.listen.start(
            targets=DataPacketUrls.data_query_base, method='POST', res_type='XHR'
        )
        func()
        target__data_packet = None
        for packet in page.listen.steps(timeout=_timeout):
            req_params = packet.request.params
            if req_params.get('type') != data_type:
                continue

            target__data_packet = packet
            break

        return target__data_packet

    def get__liveroom_list(
        self,
        date_range: list[str] | list[list[str]],
        timeout: float = None,
        throw_err=True,
    ):
        """
        获取直播间详情数据列表

        Args:
            date_range: 日期范围, 例如 `[2025-06-01, 2025-06-30]`或者`[[2025-06-01, 2025-06-15], [2025-06-16, 2025-06-30]]`
            throw_err: 在执行过程中如果出错了是否抛出异常, 如果 False 则不抛出异常并将正确获取到的数据返回
        Returns:
            数据列表. `[{begin_date, end_date, data}]`
        """

        _timeout = timeout if isinstance(timeout, (float, int)) else self._timeout

        page = self._browser.chromium.new_tab()
        if not self._wait__data_query(
            page=page,
            data_type=DataPacketType.LIVE_LIST_DETAIL.value,
            func=lambda: page.get(Urls.live_list),
            timeout=_timeout,
        ):
            raise TimeoutError('进入页面后直播间信息列表数据包获取超时')

        def catch_data(begin_date: str, end_date: str):
            # 选择自定义日期范围
            data_packet = self._wait__data_query(
                page=page,
                data_type=DataPacketType.LIVE_LIST_DETAIL.value,
                func=lambda: pick__daterange(page, begin_date, end_date),
                timeout=_timeout,
            )
            if not data_packet:
                raise TimeoutError('选择自定义日期后获取数据包超时')

            resp_data = DataPacketProcessor(data_packet).filter(
                [
                    'data[0].count',
                    'data[0].data',
                    'data[0].meta.dimensions',
                    'data[0].meta.metrics',
                    'data[0].extra',
                ]
            )
            if not resp_data.get('count'):
                raise ValueError('直播间列表为空')

            # ========== 仅作日期判断 ==========
            extra = resp_data.get('extra')
            if not isinstance(extra, dict):
                raise ValueError('直播间扩展信息格式非预期的 dict')
            if (extra_start_date := extra.get('startDateDtm')) != begin_date or (
                extra_end_date := extra.get('endDateDtm')
            ) != end_date:
                raise ValueError(
                    f'直播间扩展信息的日期 [{extra_start_date}~{extra_end_date}] 与预期日期不一致'
                )
            # ========== 仅作日期判断 ==========

            field_arr: list[dict] = [
                *resp_data.get('dimensions'),
                *resp_data.get('metrics'),
            ]
            field_dict = {item['key']: item['name'] for item in field_arr}

            data_list = []
            for item in resp_data.get('data'):
                record = {}
                for field_key, field_name in field_dict.items():
                    if field_key not in item:
                        continue
                    record[field_name] = item[field_key]['value']

                decimal_fields = [
                    '商品点击率（次数）',
                    '商品点击率（人数）',
                    '观看支付率',
                    '支付转化率',
                    '退款率',
                ]
                record = DictUtils.dict_format__ratio(record, decimal_fields)
                record = DictUtils.dict_format__round(record, decimal_fields)
                data_list.append(record)

            return data_list

        date_range_list = (
            [date_range]
            if isinstance(date_range, list)
            and len(date_range) == 2
            and all(map(lambda x: isinstance(x, str), date_range))
            else date_range
        )
        data_set: list[dict] = []
        for begin_date, end_date in date_range_list:
            try:
                data_list = catch_data(begin_date, end_date)
                data_set.append(
                    {'begin_date': begin_date, 'end_date': end_date, 'data': data_list}
                )
            except Exception as err:
                if throw_err is True:
                    raise RuntimeError(
                        f'{begin_date} ~ {end_date} 获取数据出错了: {err}'
                    ) from err

        page.close()

        return data_set

    def download__liveroom_list(
        self,
        date_range: list[str] | list[list[str]],
        save_path: str = None,
        format=True,
        timeout: float = None,
        throw_err=True,
    ):
        """
        下载直播间详情数据文件

        Args:
            date_range: 日期范围, 例如 `[2025-06-01, 2025-06-30]`或者`[[2025-06-01, 2025-06-15], [2025-06-16, 2025-06-30]]`
            save_path: 下载文件保存位置, 留空则下载到系统缓存目录下
            format: 格式化. 如果设置 True 则返回文件的路径, 否则返回文件内容
            timeout: 超时时间
            throw_err: 在执行过程中如果出错了是否抛出异常, 如果 False 则不抛出异常并将正确获取到的数据返回
        """

        _timeout = timeout if isinstance(timeout, (float, int)) else self._timeout

        page = self._browser.chromium.new_tab()
        if not self._wait__data_query(
            page=page,
            data_type=DataPacketType.LIVE_LIST_DETAIL.value,
            func=lambda: page.get(Urls.live_list),
            timeout=_timeout,
        ):
            raise TimeoutError('进入页面后直播间信息列表数据包获取超时')

        def download_file(begin_date: str, end_date: str):
            # 选择自定义日期范围
            data_packet = self._wait__data_query(
                page=page,
                data_type=DataPacketType.LIVE_LIST_DETAIL.value,
                func=lambda: pick__daterange(page, begin_date, end_date),
                timeout=_timeout,
            )
            if not data_packet:
                raise TimeoutError('选择自定义日期后获取数据包超时')

            resp_data = DataPacketProcessor(data_packet).filter(
                ['data[0].count', 'data[0].extra']
            )
            if not resp_data.get('count'):
                raise ValueError('直播间列表为空')

            # ========== 仅作日期判断 ==========
            extra = resp_data.get('extra')
            if not isinstance(extra, dict):
                raise ValueError('直播间扩展信息格式非预期的 dict')
            if (extra_start_date := extra.get('startDateDtm')) != begin_date or (
                extra_end_date := extra.get('endDateDtm')
            ) != end_date:
                raise ValueError(
                    f'直播间扩展信息的日期 [{extra_start_date}~{extra_end_date}] 与预期日期不一致'
                )
            # ========== 仅作日期判断 ==========

            download_btn = page.ele('t:button@@text()=下载数据', timeout=2)
            if not download_btn:
                raise RuntimeError('未找到 [下载数据] 按钮')

            _save_path = save_path if save_path else gettempdir()
            download_mission = download_btn.click.to_download(
                save_path=_save_path, by_js=True
            )
            file_path = download_mission.wait()
            return file_path

        date_range_list = (
            [date_range]
            if isinstance(date_range, list)
            and len(date_range) == 2
            and all(map(lambda x: isinstance(x, str), date_range))
            else date_range
        )
        data_list: list[dict] = []
        for begin_date, end_date in date_range_list:
            try:
                file_path = download_file(begin_date, end_date)
                data_list.append(
                    {'begin_date': begin_date, 'end_date': end_date, 'data': file_path}
                )
            except Exception as err:
                if throw_err is True:
                    raise RuntimeError(
                        f'{begin_date} ~ {end_date} 下载出错了: {err}'
                    ) from err

        page.close()

        if format is True:
            for i, item in enumerate(data_list):
                file_path = item.get('data')
                if not file_path or not path.exists(file_path):
                    item[i]['data'] = None
                    continue

                data = OsTools.xlsx_read(file_path)
                OsTools.file_remove(file_path)
                data_list[i]['data'] = data

        return data_list



"""
数据中心模块
"""

from BrowserAutomationLauncher import Browser

from .live import Live


class DataCenter:
    def __init__(self, browser: Browser):
        self._live = None

        self._browser = browser

    @property
    def live(self):
        """直播数据采集"""

        if not self._live:
            self._live = Live(self._browser)

        return self._live



"""
账号登录工具集
"""

from contextlib import suppress
from time import sleep
from typing import Callable

from BrowserAutomationLauncher import Browser
from DrissionPage._pages.mix_tab import MixTab


class Urls:
    home = 'https://ark.xiaohongshu.com/app-system/home'


class DataPacketUrls:
    seller_info = 'https://ark.xiaohongshu.com/api/edith/seller/info/v2'


class LoginUtils:
    def __init__(self, browser: Browser):
        self._browser = browser

    def __wait__seller_info_datapacket(self, page: MixTab, func: Callable):
        """等待用户信息数据包"""

        page.listen.start(
            targets=DataPacketUrls.seller_info, method='GET', res_type='XHR'
        )
        func()
        datapacket = page.listen.wait(timeout=12)
        if not datapacket:
            raise TimeoutError('用户信息数据包获取超时, 可能接口发生变更')

        return datapacket

    def by_cookie(
        self,
        cookie: list[dict],
        local_storage: dict = None,
        session_storage: dict = None,
        clear_before=False,
    ):
        """
        通过 Cookie 登录

        Args:
            clear_before: 是否再登录之前清除本地信息
        """

        page = self._browser.chromium.latest_tab

        with suppress(TimeoutError):
            # 即使数据包获取超时也尝试设置 cookie
            datapacket = self.__wait__seller_info_datapacket(
                page, lambda: page.get(Urls.home)
            )
            # 如果未登录, status_code 为 401
            if datapacket.response.status == 200:
                return

        if clear_before is True:
            page.clear_cache(
                session_storage=True, local_storage=True, cookies=True, cache=True
            )
            sleep(0.5)

        page.set.cookies(cookie)

        if isinstance(local_storage, dict):
            for key, value in local_storage.items():
                page.set.local_storage(key, value)

        if isinstance(session_storage, dict):
            for key, value in session_storage.items():
                page.set.session_storage(key, value)

        sleep(0.5)
        datapacket = self.__wait__seller_info_datapacket(page, lambda: page.refresh())
        if datapacket.response.status == 401:
            raise RuntimeError('账号登录失败')

        return datapacket.response.body



"""
Copyright (c) 2025-now Martian Bugs All rights reserved.
Build Date: 2025-07-01
Author: Martian Bugs
Description: 数据采集器
"""

from BrowserAutomationLauncher import BrowserInitOptions, Launcher

from ._login import LoginUtils
from .datacenter.datacenter import DataCenter


class Collector:
    """采集器. 使用之前请先调用 `connect_browser` 方法连接浏览器."""

    def __init__(self):
        self._browser = None
        self._datacenter = None
        self._login = None

        self._browser_launcher = Launcher()

    def connect_browser(self, port: int):
        """
        连接浏览器

        Args:
            port: 浏览器调试端口号
        """

        browser_options = BrowserInitOptions()
        browser_options.set_basic_options(port=port)
        browser_options.set_window_loc(width=1400, height=900)

        self._browser = self._browser_launcher.init_browser(browser_options)

    @property
    def login(self):
        """登录工具集"""

        if not self._login:
            self._login = LoginUtils(self._browser)

        return self._login

    @property
    def datacenter(self):
        """数据中心数据采集"""

        if not self._datacenter:
            self._datacenter = DataCenter(self._browser)

        return self._datacenter



from time import sleep

from DrissionPage._pages.mix_tab import MixTab


def pick__daterange(page: MixTab, begin_date: str, end_date: str):
    """
    选择自定义日期范围

    Args:
        begin_date: 起始日期, YYYY-MM-DD
        end_date: 结束日期, YYYY-MM-DD
    """

    trigger_btn = page.ele('t:button@@text()=自定义', timeout=3)
    if not trigger_btn:
        raise RuntimeError('未找到自定义日期范围按钮')
    trigger_btn = trigger_btn.parent()

    trigger_btn.set.style('position', 'fixed')
    trigger_btn.set.style('top', 0)
    trigger_btn.set.style('left', 0)
    trigger_btn.set.style('zIndex', 1001)

    for _ in range(3):
        sleep(0.5)
        trigger_btn.click()
        popup_container = trigger_btn.ele('c:div[data-popper-placement]', timeout=3)
        if popup_container:
            break
    else:
        raise RuntimeError('未找到日期选择器弹出层元素')

    year_btn_eles = popup_container.eles('x://div[contains(text(), " 年")]', timeout=1)
    if not year_btn_eles:
        raise RuntimeError('日期选择器中未找到 [年份] 切换按钮')

    # 分割传入的日期
    daterange_split = list(map(lambda x: x.split('-'), [begin_date, end_date]))

    # 判断年份
    for i, (year, month, _) in enumerate(daterange_split):
        if year == year_btn_eles[i].text[:-2]:
            continue
        sleep(0.5)

        year_btn_eles[i].click(by_js=True)
        target_year_btn = popup_container.ele(
            f't:div@@class=calendar-matrixCell@@text()={year}', timeout=2
        )
        if not target_year_btn:
            raise RuntimeError(f'日期选择器中未找到年份 [{year}]')
        if target_year_btn.attr('data-disabled') == 'true':
            raise RuntimeError(f'年份 [{year}] 禁止选择, 请检查')
        target_year_btn.click(by_js=True)
        sleep(0.5)

        # 修改完年份之后必须要选择月份
        month_noprefix = month.lstrip('0')
        target_month_btn = popup_container.ele(
            f't:div@@class=calendar-matrixCell@@text()={month_noprefix}月', timeout=2
        )
        if not target_month_btn:
            raise RuntimeError(f'选择年份之后未找到对应的月份 [{month_noprefix}月]')
        target_month_btn.click(by_js=True)
        sleep(0.5)

    # 判断月份
    month_btn_eles = popup_container.eles('x://div[contains(text(), " 月")]', timeout=1)
    if not month_btn_eles:
        raise RuntimeError('日期选择器中未找到 [月份] 切换按钮')
    for i, (_, month, _) in enumerate(daterange_split):
        month_noprefix = month.lstrip('0')
        if month == month_btn_eles[i].text[:-2]:
            continue
        sleep(0.5)

        month_btn_eles[i].click(by_js=True)
        target_month_btn = popup_container.ele(
            f't:div@@class=calendar-matrixCell@@text()={month_noprefix}月', timeout=2
        )
        if not target_month_btn:
            raise RuntimeError(f'日期选择器中未找到月份 [{month_noprefix}月]')
        if target_month_btn.attr('data-disabled') == 'true':
            raise RuntimeError(f'月份 [{month_noprefix}月] 禁止选择, 请检查')
        target_month_btn.click(by_js=True)
        sleep(0.5)

    # 选择对应日期
    day_panel_eles = popup_container.eles('c:div[data-calendar="body"]', timeout=2)
    if not day_panel_eles:
        raise RuntimeError('日期选择器中未找到日期面板')
    month_btn_eles = popup_container.eles('x://div[contains(text(), " 月")]', timeout=1)
    for i in range(2):
        for year, month, day in daterange_split:
            if month != month_btn_eles[i].text[:-2]:
                break

            day_noprefix = day.lstrip('0')
            target_day_panel = day_panel_eles[i]
            target_day_btn = target_day_panel.ele(
                f't:div@@class=calendar-dayCell@@text()={day_noprefix}', timeout=1
            )

            date = f'{year}-{month}-{day}'
            if not target_day_btn:
                raise RuntimeError(f'日期选择器中未找到 [{date}] 按钮')
            if target_day_btn.attr('data-disabled') == 'true':
                raise RuntimeError(f'目标日期 [{date}] 禁止选择, 请检查')
            target_day_btn.click(by_js=True)
            sleep(0.5)
